// SPDX-License-Identifier: GPL-2.0
/*
 * kabi_ex.c
 *
 * Copyright (C) 2024 Google LLC
 *
 * Examples for kABI stability features with --stable. See kabi_ex.h
 * for details.
 */

#include "kabi_ex.h"

struct s e0;
enum e e1;

struct ex0a ex0a;
struct ex0b ex0b;
struct ex0c ex0c;

struct ex1a ex1a;
struct ex1b ex1b;
struct ex1c ex1c;

struct ex2a ex2a;
struct ex2b ex2b;
struct ex2c ex2c;

struct ex3a ex3a;
struct ex3b ex3b;
struct ex3c ex3c;

struct ex4a ex4a;

struct ex5a ex5a;
struct ex5b ex5b;

int ex6a;
