/* SPDX-License-Identifier: GPL-2.0-or-later */
/* Generic process execution definitions.
 *
 * It should be possible to use these on really simple architectures,
 * but it serves more as a starting point for new ports.
 *
 * Copyright (C) 2007 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 */
#ifndef __ASM_GENERIC_EXEC_H
#define __ASM_GENERIC_EXEC_H

#define arch_align_stack(x) (x)

#endif /* __ASM_GENERIC_EXEC_H */
