/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef _UAPI__ASM_ARM_UNISTD_H
#define _UAPI__ASM_ARM_UNISTD_H

#include <asm/unistd_32.h>

#define __NR_sync_file_range2           __NR_arm_sync_file_range

#endif /* _UAPI__ASM_ARM_UNISTD_H */
