/* SPDX-License-Identifier: LGPL-2.1+ WITH Linux-syscall-note */
/*
 * version.h
 *
 * Copyright (C) 2000 Holger Waechtler <holger@convergence.de>
 *                    for convergence integrated media GmbH
 */

#ifndef _DVBVERSION_H_
#define _DVBVERSION_H_

#define DVB_API_VERSION 5
#define DVB_API_VERSION_MINOR 12

#endif /*_DVBVERSION_H_*/
