#define UTS_MACHINE		"aarch64"
#define LINUX_COMPILE_BY	"builduser"
#define LINUX_COMPILE_HOST	"arch-nspawn-1597728"
#define LINUX_COMPILER		"aarch64-unknown-linux-gnu-gcc (GCC) 16.1.1 20260430, GNU ld (GNU Binutils) 2.46.0"
